#!/usr/bin/python
import os
import json

from lxml import etree
from jinja2 import Template, Environment, FileSystemLoader

from mdg.xmi.parse import ns, parse_uml
from mdg.xmi.validator import validate_package
from mdg import settings


def output_level_package(env, template_definition, package):
    template = env.get_template(template_definition['source'])
    filename_template = Template(template_definition['dest'])
    filename = os.path.abspath(filename_template.render(package=package))
    dirname = os.path.dirname(filename)

    if not os.path.exists(dirname):
        os.makedirs(dirname)

    # print("Writing: " + filename)
    with open(filename, 'w') as fh:
        fh.write(template.render(package=package))


def output_level_class(env, template_definition, filter_template, package):
    template = env.get_template(template_definition['source'])
    filename_template = Template(template_definition['dest'])
    for cls in package.classes:
        if filter_template is None or filter_template.render(cls=cls) == "True":
            filename = os.path.abspath(filename_template.render(cls=cls))
            dirname = os.path.dirname(filename)
            if not os.path.exists(dirname):
                os.makedirs(dirname)

            # print("Writing: " + filename)
            with open(filename, 'w') as fh:
                fh.write(template.render(cls=cls))


def output_level_enum(env, template_definition, filter_template, package):
    template = env.get_template(template_definition['source'])
    filename_template = Template(template_definition['dest'])

    for enum in package.enumerations:
        if filter_template is None or filter_template.render(enum=enum) == "True":
            filename = os.path.abspath(filename_template.render(enum=enum))
            dirname = os.path.dirname(filename)
            if not os.path.exists(dirname):
                os.makedirs(dirname)

            # print("Writing: " + filename)
            with open(filename, 'w') as fh:
                fh.write(template.render(enum=enum))


def output_level_assoc(env, template_definition, filter_template, package):
    template = env.get_template(template_definition['source'])
    filename_template = Template(template_definition['dest'])

    for assoc in package.associations:
        if filter_template is None or filter_template.render(association=assoc) == "True":
            filename = os.path.abspath(filename_template.render(association=assoc))
            dirname = os.path.dirname(filename)
            if not os.path.exists(dirname):
                os.makedirs(dirname)
            # print("Writing: " + filename)
            with open(filename, 'w') as fh:
                fh.write(template.render(association=assoc))


def output_model(package, recipie_path):
    env = Environment(loader=FileSystemLoader(settings['templates_folder']))
    print("Generating model output")
    for template_definition in settings['templates']:
        filter_template = None
        if 'filter' in template_definition.keys():
            filter_template = Template(template_definition['filter'])

        if template_definition['level'] == 'package':
            if filter_template is None or filter_template.render(package=package) == "True":
                output_level_package(env, template_definition, package)

        elif template_definition['level'] == 'class':
            output_level_class(env, template_definition, filter_template, package)

        elif template_definition['level'] == 'enumeration':
            output_level_enum(env, template_definition, filter_template, package)

        elif template_definition['level'] == 'assocication':
            output_level_assoc(env, template_definition, filter_template, package)

    for child in package.children:
        output_model(child, recipie_path)


def output_test_cases(test_cases):
    print("Generating test case output")
    for case in test_cases:
        serialised = json.dumps(serialize_instance(case), indent=2)

        for template_definition in settings['test_templates']:
            filename_template = Template(template_definition['dest'])
            filename = os.path.abspath(filename_template.render(ins=case))
            dirname = os.path.dirname(filename)
            if not os.path.exists(dirname):
                os.makedirs(dirname)
            # print("Writing: " + filename)
            with open(filename, 'w') as fh:
                fh.write(serialised)


def serialize_instance(instance):
    ret = {}

    for attr in instance.attributes:
        ret[attr.name] = attr.value

    # for assoc in instance.associations_to:
    #    if assoc.source_multiplicity[1] == '*':
    #        if assoc.source.name not in ret.keys():
    #            ret[assoc.source.name] = [serialize_instance(assoc.source),]
    #        else:
    #            ret[assoc.source.name].append(serialize_instance(assoc.source))
    #    else:
    #            ret[assoc.source.name] = serialize_instance(assoc.source)

    for assoc in instance.associations_from:
        if assoc.destination_multiplicity[1] == '*':
            if assoc.destination.name not in ret.keys():
                ret[assoc.destination.name] = [serialize_instance(assoc.destination), ]
            else:
                ret[assoc.destination.name].append(serialize_instance(assoc.destination))
        else:
            ret[assoc.destination.name] = serialize_instance(assoc.destination)

    return ret


def parse(recipie_path):

    # with open(config_filename, 'r') as config_file:
    #    settings = yaml.load(config_file.read(), Loader=yaml.SafeLoader)

    tree = etree.parse(settings['source'])
    model = tree.find('uml:Model', ns)
    root_package = model.xpath("//packagedElement[@name='%s']" % settings['root_package'], namespaces=ns)
    if len(root_package) == 0:
        print("Root packaged element not found. Settings has:{}".format(settings['root_package']))
        return
    root_package = root_package[0]

    model_package, test_cases = parse_uml(root_package, tree)
    print("Base Model Package: " + model_package.name)

    errors = validate_package(model_package)
    if len(errors) > 0:
        print("Validation Errors:")
        for error in errors:
            print("    {}".format(error))

    output_model(model_package, recipie_path)
    output_test_cases(test_cases)



import os
import yaml

settings = {
    "generation_type": "default",
}

generation_fields = {
    "default": {
        "boolean": "boolean",
        "date": "Date",
        "dateTime": "DateTime",
        "decimal": "Double",
        "enum": "String",
        "int": "int",
        "bigint": "int",
        "integer": "int",
        "long": "int",
        "string": "String",
    },
    "spring data rest": {
        "boolean": "boolean",
        "date": "Date",
        "dateTime": "DateTime",
        "decimal": "Double",
        "enum": "String",
        "int": "int",
        "bigint": "int",
        "integer": "int",
        "long": "int",
        "string": "String",
    },
    "django": {
        "boolean": "BooleanField",
        "int": "IntegerField",
        "bigint": "BigIntegerField",
        "decimal": "DecimalField",
        "string": "CharField",
        "text": "TextField",
        "duration": "DurationField",
        "file": "FileField",
        "float": "FloatField",
    },
    "marshmallow": {
        "boolean": "Boolean",
        "int": "Integer",
        "integer": "Integer",
        "bigint": "Integer",
        "decimal": "Decimal",
        "string": "String",
        "text": "Text",
        "duration": "Duration",
        "file": "File",
        "float": "Float",
        "date": "Date",
        "dateTime": "DateTime",
        "date_time": "DateTime",
    }
}

try:
    with open(os.environ.get('PYXMI_SETTINGS_MODULE', ""), 'r') as config_file:
        settings = yaml.load(config_file.read(), Loader=yaml.SafeLoader)
except FileNotFoundError:
    print("Using default settings")



import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name='pymdg',
    version='0.1a2',
    author='Paul Atkin',
    description='Model driven genration - from UML to Code & Docs',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/semprini/pyMDG',
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "lxml",
        "jinja2",
        "pyyaml",
    ],
    python_requires='>=3.5',
)



#!/usr/bin/python
from lxml import etree

from mdg.xmi.parse import ns, parse_uml
from mdg import settings


class ClassValidationError(object):
    def __init__(self, package, cls, error):
        self.package = package
        self.error = error
        self.cls = cls

    def __repr__(self):
        return "Class error: {}{} | {}".format(self.package.path, self.cls.name, self.error)


class AttributeValidationError(object):
    def __init__(self, package, cls, attr, error):
        self.package = package
        self.error = error
        self.cls = cls
        self.attr = attr

    def __repr__(self):
        return "Attribute error: {}{}.{} | {}".format(self.package.path, self.cls.name, self.attr.name, self.error)


def validate_package(package):
    errors = []

    for cls in package.classes:
        if cls.id_attribute is None:
            if cls.supertype is None:
                errors.append(ClassValidationError(package, cls, "no primary key"))
        elif cls.supertype is not None:
            if cls.supertype.id_attribute != cls.id_attribute:
                errors.append(ClassValidationError(package, cls, "primary key in both class and supertype"))

        for attr in cls.attributes:
            if attr.stereotype == "auto" and attr.type not in ("int", "bigint"):
                errors.append(AttributeValidationError(package, cls, attr, "auto increment field must be int or bigint"))

    for child in package.children:
        errors += validate_package(child)

    return errors


def validate(recipie_path):
    tree = etree.parse(settings['source'])
    model = tree.find('uml:Model', ns)
    root_package = model.xpath("//packagedElement[@name='%s']" % settings['root_package'], namespaces=ns)
    if len(root_package) == 0:
        print("Root packaged element not found. Settings has:{}".format(settings['root_package']))
        return
    root_package = root_package[0]

    model_package, test_cases = parse_uml(root_package, tree)

    # validations
    # Does each object have a primary key
    # Do objects with primary keys have a parent class which also has a primary key
    # Are all auto increment fields int
    print(validate_package(model_package))

    # Does each class have a domain
    # Are there unexpected attribute types



#!/usr/bin/python
import re


class UMLPackage(object):
    def __init__(self, id, name, parent=None, stereotype=None):
        self.classes = []
        self.associations = []
        self.children = []
        self.instances = []
        self.enumerations = []
        self.parent = parent
        self.stereotype = stereotype
        self.inherited_stereotypes = []
        self.name = name
        self.id = id
        self.diagrams = []
        self.documentation = ""

        if self.parent is None:
            self.root_package = self
            self.path = '/' + self.name + '/'
        else:
            self.root_package = parent.root_package
            self.inherited_stereotypes += parent.inherited_stereotypes
            self.path = self.parent.path + self.name + '/'

    def __str__(self):
        return f"{self.name}"

    def find_by_id(self, id):
        """ Finds UMLPackage, UMLClass, UMLEnumeration or UMLInstance object with specified Id
        Looks for classes part of this package and all sub-packages
        """
        if self.id == id:
            return self

        for cls in self.classes:
            if cls.id == id:
                return cls

        for ins in self.instances:
            if ins.id == id:
                return ins

        for enum in self.enumerations:
            if enum.id == id:
                return enum

        for child in self.children:
            res = child.find_by_id(id)
            if res is not None:
                return res


class UMLInstance(object):
    def __init__(self, package, name, id):
        self.attributes = []
        self.associations_from = []
        self.associations_to = []
        self.package = package
        self.stereotype = None
        self.name = name
        self.id = id
        self.documentation = ""


class UMLAssociation(object):
    def __init__(self, package, source, destination):
        self.package = package
        self.association_type = None

        self.source = source
        self.source_name = None
        self.source_multiplicity = ['0', '0']
        source.associations_from.append(self)

        self.destination = destination
        self.destination_multiplicity = ['0', '0']
        self.destination_name = None
        destination.associations_to.append(self)
        self.documentation = ""

    @property
    def source_name_camel(self):
        return re.sub(r'_([a-z])', lambda x: x.group(1).upper(), self.source_name)

    @property
    def destination_name_camel(self):
        return re.sub(r'_([a-z])', lambda x: x.group(1).upper(), self.destination_name)


class UMLEnumeration(object):
    def __init__(self, package, name, id):
        self.values = []
        self.package = package
        self.name = name
        self.id = id
        self.documentation = ""

    def __str__(self):
        return f"{self.name}"


class UMLClass(object):
    def __init__(self, package, name, id):
        self.name = name
        self.id = id
        self.attributes = []
        self.associations_from = []
        self.associations_to = []
        self.package = package
        self.supertype = None
        self.supertype_id = None
        self.is_supertype = False
        self.stereotypes = []
        self.id_attribute = None
        self.documentation = ""

        for inherited_stereotype, inherited_package in package.inherited_stereotypes:
            if not hasattr(self, inherited_stereotype):
                setattr(self, inherited_stereotype, inherited_package)


class UMLAttribute(object):
    def __init__(self, parent, name, id):
        self.parent = parent
        self.name = name
        self.id = id
        self.is_unique = False
        self.stereotype = None
        self.classification = None
        self.classification_id = None
        self.documentation = ""
        self.dest_type = None
        self.value = None

    def name_camel(self):
        return re.sub(r'_([a-z])', lambda x: x.group(1).upper(), self.name)
